(ns document-storage.postgres.documents
  (:require [document-storage.postgres.encoding :as encoding]))


(defn get-last-document-version [storage document-id repository]
  (if-let [frame (.run-query storage :get-last-document-frame
                               {:repository repository
                                :document-id document-id})]
    (:version frame)
    -1))

(defn load-document
  ([storage document-id repository]
   (when-let [doc (.run-query storage :get-last-document-frame
                              {:repository repository
                               :document-id document-id})]
     (encoding/decode-document (:data doc))))
  ([storage document-id version repository]
   (when-let [doc (.run-query storage :get-document-frame-by-version
                              {:repository repository
                               :version version
                               :document-id document-id})]
     (encoding/decode-document (:data doc)))))

(defn save-document [storage document-id document repository]
  (let [doc (encoding/encode-document document)]
    ;;VALUES (:uuid, :document-id, :type, :encoding, :version, :reference, :data)
    (.run-query storage :create-document-frame!
                {:repository repository
                 :document-id document-id
                 :type :ds-type/key
                 :encoding :ds-encoding/transit+json
                 :version (inc (get-last-document-version storage document-id repository))
                 :reference nil
                 :data doc})))


(defn delete-document [storage document-id repository]
    (.run-query storage :create-document-frame!
                {:repository repository
                 :document-id document-id
                 :type :ds-type/delete
                 :encoding :ds-encoding/edn
                 :version (inc (get-last-document-version storage document-id repository))
                 :reference nil
                 :data ""}))
